// Konwersja sprite'a w pliku *.BMP na dane grafiki ATARI, 4kolory, tryb znakowy
//
// output files:
//        *.bin  - lda/sta
//        *.msk  - lda/and/sta
//        *.ora  - lda/and/ora/sta
//
program BMP2DAT;
uses
       Crt;
type 
     Header = record
               Signature:        word;
               FileSize:         dword;
               reserved:         dword;
               DataOffset:       dword;
               SizeOfInfoHeader: dword;
               Width:            dword;
               Height:           dword;
               Planes:           word;
               BitCount:         word;
               Compression:      dword;
               ImageSize:        dword;
               XpixelsPerM:      dword;
               YpixelsPerM:      dword;
               ColorsUsed:       dword;
               ColorsImportant:  dword;
     end;

var
   in_file, out_file, msk_file, ora_file:           file;
   BMPHeader:                                       Header;
   BMPPalette:                                      array [0..255] of dword;
   BMPRaster:                                       array [0..65355] of byte;
   BMPRasterFlip:                                   array [0..65355] of byte;
   i,ii,error:                                      integer;
   offset,row:                                      integer;
   data0,data1,data2,data3,data4,data5,data6,data7: byte;
   SizeRaster,SizeWidth,SizeHeight:                 integer;
   ATARIHeader:                                     integer;
   file_param, file_name, file_ext:                 string;
   bits76, bits54, bits32, bits10:                  byte;

begin
     if ParamCount = 0 then begin
        WriteLn('Nie podano pliku do konwersji!');
        WriteLn;
        WriteLn('Skadnia uycia: bmp2dat plik_wejciowy');
        Exit;
     end;

     file_param:= ParamStr(1);
     file_name:=  Copy(file_param,1,Length(file_param)-4);
     file_ext:=   Copy(file_param,Length(file_param)-2,3);

     Assign(in_file, ParamStr(1));
     Reset(in_file, 1);                      { Record size = 1 }

     //Assign(out_file, file_name + '.bin');
     //Rewrite(out_file, 1);
     Assign(msk_file, file_name + '.msk');
     Rewrite(msk_file, 1);
     Assign(ora_file, file_name + '.ora');
     Rewrite(ora_file, 1);

     BlockRead(in_file,BMPHeader,sizeof(BMPHeader)); Writeln('BMPHeader ', sizeof(BMPHeader), ' bytes.');
     BlockRead(in_file,BMPPalette,sizeof(BMPPalette)); Writeln('BMPPalette ', sizeof(BMPPalette), ' bytes.');
     {$I-} 
     BlockRead(in_file,BMPRaster,BMPHeader.Width * BMPHeader.Height);
     error := IOResult;
     if error = 0 then WriteLn('BMPRaster ', sizeof(BMPRaster), ' bytes.') else WriteLn('BMPRaster - Load with error! ');
     {$I+}

     if BMPHeader.BitCount <> 8 then begin WriteLn('This BMP file has not 256 colors!'); Halt; end;

     // odwrcenie kolejnoci bajtw w kadej linii bitmapy
     for ii:=0 to BMPHeader.Height-1 do begin
         for i:=0 to BMPHeader.Width-1 do begin
             BMPRasterFlip[(BMPHeader.Width*BMPHeader.Height)-BMPHeader.Width+i-ii*BMPHeader.Width]:= BMPRaster[i+ii*BMPHeader.Width];
         end;
     end;

     // konwersja 4 bajtw koloru na 1 bajt koloru w ATARI
     for i:=0 to (BMPHeader.Width * BMPHeader.Height)-1 do begin
         data0:= (BMPRasterFlip[i+3] and $03); //if data0 = %10 then data0:= %11;
         data1:= (BMPRasterFlip[i+2] and $03); //if data1 = %10 then data1:= %11;
         data2:= (BMPRasterFlip[i+1] and $03); //if data2 = %10 then data2:= %11;
         data3:= (BMPRasterFlip[i+0] and $03); //if data3 = %10 then data3:= %11;

         BMPRaster[i div 4]:= data3 << 6 + data2 << 4 + data1 << 2 + data0;

         i:= i+3;
     end;

     SizeRaster:= BMPHeader.Width * BMPHeader.Height;
     SizeWidth:=  BMPHeader.Width div 4;
     SizeHeight:= BMPHeader.Height div 8;
     ii:= 0;
     offset:= 0;
     ATARIHeader:= 0; //3;
     //BMPRasterFlip[0]:= SizeWidth;               // szeroko SPRITE w bajtach
     //BMPRasterFlip[1]:= SizeHeight;              // wysoko SPRITE  w bajtach
     //BMPRasterFlip[2]:= SizeWidth*SizeHeight;    // rozmiar SPRITE  w bajtach

     // konwersja z bitmapy na tryb znakowy, 1znak = 8bajtw
     for row:=0 to SizeHeight-1 do begin
         for i:=0 to SizeWidth-1 do begin
             data0:= BMPRaster[i+SizeWidth*0+offset];
             data1:= BMPRaster[i+SizeWidth*1+offset];
             data2:= BMPRaster[i+SizeWidth*2+offset];
             data3:= BMPRaster[i+SizeWidth*3+offset];
             data4:= BMPRaster[i+SizeWidth*4+offset];
             data5:= BMPRaster[i+SizeWidth*5+offset];
             data6:= BMPRaster[i+SizeWidth*6+offset];
             data7:= BMPRaster[i+SizeWidth*7+offset];
             BMPRasterFlip[ii+0+ATARIHeader]:= data0;
             BMPRasterFlip[ii+1+ATARIHeader]:= data1;
             BMPRasterFlip[ii+2+ATARIHeader]:= data2;
             BMPRasterFlip[ii+3+ATARIHeader]:= data3;
             BMPRasterFlip[ii+4+ATARIHeader]:= data4;
             BMPRasterFlip[ii+5+ATARIHeader]:= data5;
             BMPRasterFlip[ii+6+ATARIHeader]:= data6;
             BMPRasterFlip[ii+7+ATARIHeader]:= data7;
             ii:= ii+8;
         end;
         offset:= offset + 8*SizeWidth;
     end;

     //BlockWrite(out_file,BMPRasterFlip,(SizeRaster div 4)+ATARIHeader);
     BlockWrite(ora_file,BMPRasterFlip,(SizeRaster div 4)+ATARIHeader);

     // prepare mask form binary
     for i:=0 to SizeOf(BMPRasterFlip) do begin
          data0:= BMPRasterFlip[i];
         bits76:= data0 and %11000000; if (bits76 = %01000000) or (bits76 = %10000000) then bits76:= %11000000;
         bits54:= data0 and %00110000; if (bits54 = %01000000) or (bits54 = %10000000) then bits54:= %00110000;
         bits32:= data0 and %00001100; if (bits32 = %01000000) or (bits32 = %10000000) then bits32:= %00001100;
         bits10:= data0 and %00000011; if (bits10 = %01000000) or (bits10 = %10000000) then bits10:= %00000011;
          data1:= bits76 + bits54 + bits32 + bits10;
          data1:= data1 xor $ff;
         BMPRaster[i]:= data1;
     end;

     BlockWrite(msk_file,BMPRaster,(SizeRaster div 4)+ATARIHeader);

     close(in_file);
     //close(out_file);
     close(msk_file);
     close(ora_file);
end.
